import java.util.*;
import java.util.concurrent.*;

public class FortressSolution {
    static class Solution {
        private ArrayList<Integer>[] adj;
        private BitSet protected_nodes;
        private int[] queue;
        private int[] distances;
        private boolean[] visited;
        
        @SuppressWarnings("unchecked")
        public Set<Integer> solve(int n, int m, int[][] roads, int[][] guards) {
            adj = new ArrayList[n + 1];
            queue = new int[n + 1];
            distances = new int[n + 1];
            visited = new boolean[n + 1];
            protected_nodes = new BitSet(n + 1);
            
            for (int i = 1; i <= n; i++) {
                adj[i] = new ArrayList<>(4);
            }
        
            for (int[] road : roads) {
                adj[road[0]].add(road[1]);
                adj[road[1]].add(road[0]);
            }
            
            if (n > 50000 && Runtime.getRuntime().availableProcessors() > 1) {
                processGuardsParallelOptimized(guards, n);
            } else {
                processGuardsSequential(guards, n);
            }
            
            Set<Integer> result = new HashSet<>(n);
            for (int i = protected_nodes.nextSetBit(0); i >= 0; i = protected_nodes.nextSetBit(i + 1)) {
                result.add(i);
            }
            return result;
        }
        
        private void processGuardsSequential(int[][] guards, int n) {
            for (int[] guard : guards) {
                bfsOptimized(guard[0], guard[1], n);
            }
        }
        
        private void processGuardsParallelOptimized(int[][] guards, int n) {
            int processors = Runtime.getRuntime().availableProcessors();
            ExecutorService executor = Executors.newFixedThreadPool(processors);
            
            try {
                List<Future<?>> futures = new ArrayList<>(guards.length);
                for (int[] guard : guards) {
                    final int start = guard[0];
                    final int range = guard[1];
                    futures.add(executor.submit(() -> bfsOptimized(start, range, n)));
                }
                
                for (Future<?> future : futures) {
                    future.get();
                }
            } catch (InterruptedException | ExecutionException e) {
                Thread.currentThread().interrupt();
            } finally {
                executor.shutdown();
            }
        }
        
        private void bfsOptimized(int start, int range, int n) {
            Arrays.fill(visited, false);
            int qHead = 0, qTail = 0;
            
            queue[qTail++] = start;
            visited[start] = true;
            distances[start] = 0;
            
            while (qHead < qTail) {
                int node = queue[qHead++];
                
                if (distances[node] > range) break;
                
                synchronized (protected_nodes) {
                    protected_nodes.set(node);
                }
                
                for (int next : adj[node]) {
                    int newDistance = distances[node] + 1;
                    if (!visited[next] && newDistance <= range) {
                        visited[next] = true;
                        distances[next] = newDistance;
                        queue[qTail++] = next;
                    }
                }
            }
        }
    }

    public static void main(String[] args) {
        Scanner scanner = new Scanner(System.in);
        int n = scanner.nextInt();
        int m = scanner.nextInt();
        int k = scanner.nextInt();
        
        int[][] roads = new int[m][2];
        for (int i = 0; i < m; i++) {
            roads[i][0] = scanner.nextInt();
            roads[i][1] = scanner.nextInt();
        }
        
        int[][] guards = new int[k][2];
        for (int i = 0; i < k; i++) {
            guards[i][0] = scanner.nextInt();
            guards[i][1] = scanner.nextInt();
        }
        
        Solution solution = new Solution();
        Set<Integer> result = solution.solve(n, m, roads, guards);
        
        System.out.println(result.size());
        if (!result.isEmpty()) {
            ArrayList<Integer> sorted = new ArrayList<>(result);
            Collections.sort(sorted);
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < sorted.size(); i++) {
                sb.append(sorted.get(i));
                if (i < sorted.size() - 1) {
                    sb.append(' ');
                }
            }
            System.out.println(sb);
        } else {
            System.out.println();
        }
        
        scanner.close();
    }
}
