#include <iostream>
#include <vector>
#include <queue>
#include <set>
#include <algorithm>

using namespace std;

class Solution {
private:
    vector<vector<int>> adj;  
    vector<int> guard_pos;    
    vector<int> guard_power;  
    int n;                    
    
    vector<int> bfs(int start) {
        vector<int> dist(n + 1, -1);
        queue<int> q;
        q.push(start);
        dist[start] = 0;
        
        while (!q.empty()) {
            int curr = q.front();
            q.pop();
            
            for (int next : adj[curr]) {
                if (dist[next] == -1) {
                    dist[next] = dist[curr] + 1;
                    q.push(next);
                }
            }
        }
        return dist;
    }

public:
    void solve() {
        int m, k;
        cin >> n >> m >> k;
        
        adj.resize(n + 1);
        
        for (int i = 0; i < m; i++) {
            int a, b;
            cin >> a >> b;
            adj[a].push_back(b);
            adj[b].push_back(a);
        }
        
        guard_pos.resize(k);
        guard_power.resize(k);
        for (int i = 0; i < k; i++) {
            cin >> guard_pos[i] >> guard_power[i];
        }
        
        set<int> protected_warehouses;
        
        for (int i = 0; i < k; i++) {
            vector<int> distances = bfs(guard_pos[i]);
            
            for (int j = 1; j <= n; j++) {
                if (distances[j] != -1 && distances[j] <= guard_power[i]) {
                    protected_warehouses.insert(j);
                }
            }
        }
        
        cout << protected_warehouses.size() << endl;
        if (!protected_warehouses.empty()) {
            for (int warehouse : protected_warehouses) {
                cout << warehouse << " ";
            }
            cout << endl;
        } else {
            cout << endl;
        }
    }
};

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    
    Solution solution;
    solution.solve();
    
    return 0;
}
